@@ -904,6 +904,10 @@ class Application(object):
             raise NEOStorageError('Transaction %r not found' % (tid, ))
         return (txn_info, txn_ext)
 
+    # XXX: The following 2 methods fail when they reconnect to a storage after
+    #      they already sent a request to a previous storage.
+    #      See also testStorageReconnectDuringXxx
+
     def undoLog(self, first, last, filter=None, block=0):
         # XXX: undoLog is broken
         if last < 0:
@@ -981,34 +985,30 @@ class Application(object):
             append(txn_info)
         return (tid, txn_list)
 
-    def history(self, oid, version=None, size=1, filter=None):
-        queue = self._getThreadQueue()
+    def history(self, oid, size=1, filter=None):
         # Get history informations for object first
         packet = Packets.AskObjectHistory(oid, 0, size)
         for node, conn in self.cp.iterateForObject(oid, readable=True):
             try:
-                conn.ask(packet, queue=queue)
+                history_list = self._askStorage(conn, packet)
             except ConnectionClosed:
                 continue
-        history_dict = {}
-        self.waitResponses(queue, history_dict)
-        # Now that we have object informations, get txn informations
-        history_list = []
-        append = history_list.append
-        for serial in sorted(history_dict.keys(), reverse=True):
-            size = history_dict[serial]
-            txn_info, txn_ext = self._getTransactionInformation(serial)
-            # create history dict
-            txn_info.pop('id')
-            txn_info.pop('oids')
-            txn_info.pop('packed')
-            txn_info['tid'] = serial
-            txn_info['version'] = ''
-            txn_info['size'] = size
-            if filter is None or filter(txn_info):
-                append(txn_info)
-            self._insertMetadata(txn_info, txn_ext)
-        return history_list
+            # Now that we have object informations, get txn informations
+            result = []
+            # history_list is already sorted descending (by the storage)
+            for serial, size in history_list:
+                txn_info, txn_ext = self._getTransactionInformation(serial)
+                # create history dict
+                txn_info.pop('id')
+                txn_info.pop('oids')
+                txn_info.pop('packed')
+                txn_info['tid'] = serial
+                txn_info['version'] = ''
+                txn_info['size'] = size
+                if filter is None or filter(txn_info):
+                    result.append(txn_info)
+                self._insertMetadata(txn_info, txn_ext)
+            return result
 
     @profiler_decorator
     def importFrom(self, source, start, stop, tryToResolveConflict):
